@@ -5,7 +5,7 @@ use warnings;
 use Want qw(want);
 our $DEBUG;
 
-our $VERSION = '0.02';
+our $VERSION = '0.04';
 
 sub logger {
     my ($self, $msg, $class) = @_;
@@ -25,27 +25,28 @@ AUTOLOAD {
     $module =~ s/=.*//;
     my $class = "Pipe::Tube::" . ucfirst $module;
     $self->logger("AUTOLOAD: '$AUTOLOAD', module: '$module', class: '$class'");
+    ## no critic (ProhibitStringyEval)
     eval "use $class";
     die "Could not load '$class' $@\n" if $@;
 
     if ($self eq "Pipe") {
         $self = bless {}, "Pipe";
     }
-    my $last_thingy = (want('VOID') or want('LIST') or (want('SCALAR') and not want('OBJECT')) ? 1 : 0);
+    #my $last_thingy = (want('VOID') or want('LIST') or (want('SCALAR') and not want('OBJECT')) ? 1 : 0);
     $self->logger("context: $_: " . want($_)) for (qw(VOID SCALAR LIST OBJECT));
 
     $self->logger("params: " . join "|", @_);
     my $obj = $class->new(@_);
     push @{ $self->{Pipe} }, $obj;
 
-    if ($last_thingy) {
-        $self->logger("last thingy");
-        return $self->run_pipe;
-    }
+    #if ($last_thingy) {
+    #    $self->logger("last thingy");
+    #    return $self->run_pipe;
+    #}
     return $self;
 }
 
-sub run_pipe {
+sub run {
     my ($self) = @_;
     $self->logger("Pipe::run_pipe called");
     return if not @{ $self->{Pipe} };
@@ -94,12 +95,12 @@ Pipe - Framework to create pipes using iterators
 =head1 SYNOPSIS
 
  use Pipe;
- my @input = Pipe->cat("t/data/file1", "t/data/file2");
- my @lines = Pipe->cat("t/data/file1", "t/data/file2")->chomp;
- my @uniqs = Pipe->cat("t/data/file1", "t/data/file2")->chomp->uniq;
-
- Pipe->cat("t/data/file1", "t/data/file2")->uniq->print("t/data/out");
+ my @input = Pipe->cat("t/data/file1", "t/data/file2")->run;
+ my @lines = Pipe->cat("t/data/file1", "t/data/file2")->chomp->run;
+ my @uniqs = Pipe->cat("t/data/file1", "t/data/file2")->chomp->uniq->run;
 
+ my $pipe = Pipe->cat("t/data/file1", "t/data/file2")->uniq->print("t/data/out");
+ $pipe->run;
 
 
 =head1 WARNING
@@ -110,34 +111,58 @@ This is Alpha version. The user API might still change
 
 Building an iterating pipe with prebuilt and home made tubes.
 
+=head2 Methods
+
+=over 4
+
+=item logger
+
+Method to print something to the log file, especially for debugging
+This method is here to be use by Tube authors
+
+    $self->logger("log messages");
+
+=item run
+
+The method that actually executes the whole pipe.
+
+my $pipe  = Pipe->cat("file");
+$pipe->run;
+
+=back
+
+=head2 Tubes
+
 Tubes available in this distibution:
 
-=head2 cat
+=over 4
+
+=item cat
 
 Read in the lines of one or more file.
 
-=head2 chomp
+=item chomp
 
 Remove trailing newlines from each line.
 
 
-=head2 find
+=item find
 
 Pipe->find(".")
 
 Returns every file, directory, etc. under the directory tree passed to it.
 
-=head2 for
+=item for
 
 Pipe->for(@array)
 
 Iterates over the elements of an array. Basically the same as the for or foreach loop of Perl.
 
-=head2 glob
+=item glob
 
 Implements the Perl glob function.
 
-=head2 grep
+=item grep
 
 Selectively pass on values.
 
@@ -154,16 +179,14 @@ Very similar to the built-in grep command of Perl but instead of regex
 you have to pass a compiled regex using qr// and instead of a block you
 have to pass an anonymous   sub {}
 
-=head2 map
+=item map
 
 Similar to the Perl map construct, except that instead of a block you pass
 an anonymous function sub {}.
 
  ->map(  sub {  length $_[0] } );
 
-=head2 print
-
-TODO Not implemented yet
+=item print
 
 Prints out its input.
 By default it prints to STDOUT but the user can supply a filename or a filehandle.
@@ -172,12 +195,12 @@ By default it prints to STDOUT but the user can supply a filename or a filehandl
  Pipe->cat("t/data/file1", "t/data/file2")->print("out.txt");
  Pipe->cat("t/data/file1", "t/data/file2")->print(':a', "out.txt");
 
-=head2 say
+=item say
 
 It is the same as print but adds a newline at the end of each line.
 The name is Perl6 native.
 
-=head2 sort
+=item sort
 
 Similar to the built in sort function of Perl. As sort needs to have all 
 the data in the memory, once you use sort in the Pipe it stops being
@@ -188,7 +211,18 @@ sorting function. The two values to be compared are passed to this function.
 
  Pipe->cat("t/data/numbers1")->chomp->sort( sub { $_[0] <=> $_[1] } );
 
-=head2 tuple
+=item split
+
+Given a regex (or a simple string), will split all the incoming strings and return
+an array reference for each row.
+
+Param: string or regex using qr//
+
+Input: string(s)
+
+Output: array reference(s)
+
+=item tuple
 
 Given one or more array references, on every iteration it will return an n-tuple
 (n is the number of arrays), one value from each source array.
@@ -206,7 +240,7 @@ Input: disregards any input so it can be used as a starting element of a Pipe
 
 Ouput: array refs of n elements
 
-=head2 uniq
+=item uniq
 
 Similary to the unix uniq command eliminate duplicate consecutive values.
 
@@ -215,6 +249,7 @@ Similary to the unix uniq command eliminate duplicate consecutive values.
 Warning: as you can see from the example this method does not give real unique
 values, it only eliminates consecutive duplicates.
 
+=back
 
 =head1 Building your own tube
 
@@ -223,21 +258,23 @@ Pipe::Tube::Thing that inherits from Pipe::Tube, our abstract Tube.
 
 Implement one or more of these methods in your subclass as you please.
 
-=head2 init
+=over 4 
+
+=item init
 
 Will be called once when initializing the pipeline. 
 It will get ($self, @args)  where $self is the Pipe::Tube::Thing object
 and @args are the values given as parameters to the ->thing(@args) call
 in the pipeline.
 
-=head2 run
+=item run
 
 Will be called every time the previous tube in the pipe returns one or more values.
 It can return a list of values that will be passed on to the next tube.
 If based on the current state of Thing there is nothing to do you should call
 return; with no parameters.
 
-=head2 finish
+=item finish
 
 Will be called once when the Pipe Manager notices that this Thing should be finished.
 This happens when Thing is the first active element in the pipe (all the previous tubes
@@ -247,6 +284,8 @@ The finish() method should return a list of values that will be passed on to the
 tube in the pipe. This is especially useful for Tubes such as sort that can to their thing
 only after they have received all the input.
 
+=back
+
 =head2 Debugging your tube
 
 You can call $self->logger("some message") from your tube. 
@@ -359,7 +398,7 @@ to Gaal Yahas
 
 Gabor Szabo <gabor@pti.co.il>
 
-=head1 Copyright
+=head1 COPYRIGHT
 
 Copyright 2006 by Gabor Szabo <gabor@pti.co.il>.
 
@@ -369,7 +408,7 @@ See http://www.perl.com/perl/misc/Artistic.html
 
 =head1 See Also
 
-L<Shell::Autobox>
+L<Shell::Autobox> and L<File::Tools>
 
 
 =cut
@@ -421,7 +460,7 @@ L<Shell::Autobox>
 #     print with \n added like in Perl6 but with optional ("filename") to print to that file
 #     without explicitely opening it.
 #     
-#=head2 flat
+#=item flat
 
 #Will flatten a pipe. I am not sure it is useful at all.
 #The issue is that most of the tubes are iterators but "sort" needs to collect all the inputs
@@ -444,6 +483,16 @@ L<Shell::Autobox>
 # 
 #  split up the input stream and have more than one tails
 #
+
+# A tube might need to be able to terminate itself (or the whole pipe ?) without calling exit or die.
+#   We might allow any tube to tell the pipe to skip any further call to it.
+#   Or it can just decide it will keep calling  return; on every call except in finish() ?
+# 
+#
+
+# Trim
+
+# TODO: add 3rd parameter of split
  
 1;
 
